/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#include "../lib/piconf/piconfs/piconfs.hpp"
#include "../lib/piconf/piconfs/tag/tcp_tag.hpp"
#include "../lib/piconf/piconfs/tag/conf_tag.hpp"

#include <iostream>
#include <cstring>
#include <unistd.h>

int main(int argc, char** argv) {
	try {
		std::string file = piconf::parse_args(piconf::name, argc, argv);
		VIO::TCP timer;
		VIO::IO::timer = &timer;
		timespec span = {0, 10000000};
		VIO::TCP::forkTimer(&timer, span);
		pid_t pid;
		if (0 > (pid = fork()))
			throw (std::string)"Cannot fork process";
		if (pid) {
			std::cout << pid << std::endl;
			return piconf::error.get();
		}
		piconf::tcp_tag tcp(file.c_str());
		for (bool undone = true; undone; undone = tcp.process());
	}
	catch (VIO::IO::Error& error) {
		piconf::error.set(piconf::error_t::SYSCALL_ERROR);
		std::cerr << error.what() << std::endl;
	}
	catch (std::bad_alloc& error) {
		piconf::error.set(piconf::error_t::SYSCALL_ERROR);
		std::cerr << "No RAM left" << std::endl;
	}
	catch (std::string& error) {
		if (!piconf::error.get())
			piconf::error.set(piconf::error_t::PARSING_ERROR | piconf::error_t::PICONFS_ERROR);
		std::cerr << error << std::endl;
	}
	return piconf::error.get();
}
