/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _INTERIM_H
#define _INTERIM_H

#include "../vio/FD.hpp"

#include <iostream>
#include <stdio.h>
#include <vector>
#include <deque>
#include <ncurses.h>

#define KEY_ESC 27

class interim {
public:
	typedef int nkey_t;
	typedef std::vector<nkey_t> nkeys_t;
	VIO::FD in;
private:
	struct pos_t {
		size_t y;
		size_t x;
		pos_t();
	};
	struct win_t {
		pos_t pos;
		pos_t max;
	};
	win_t win, scr;
	typedef std::string line_t;
	typedef std::deque<line_t> lines_t;
	typedef std::vector<unsigned char> keys_t;
	size_t rows, top;
	keys_t keys;
	keys_t::iterator key;
	size_t left;
	typedef uint8_t state_t;
	static const state_t
	TOPSTART = 0x01,
	AUTOMOVE = 0x02,
	LINEDOWN = 0x04;
	state_t state;
	keys_t keystroke();
	void dimensions();
	void move(int nkey);
public:
	lines_t lines;
	interim();
	void topstart(bool what = true);
	void automove(bool what = true);
	void open();
	void close();
	void update(bool force = true);
	nkeys_t parseKeys();
	~interim();
};

#endif
