/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_H
#define _PICONF_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __STDC_FORMAT_MACROS
#include <inttypes.h>
#include <vector>
#include <deque>
#include <string>
#include <sstream>
#include <iostream>
#include <time.h>

#include "../../vio/IO.hpp"

#define PICONF_ROOT "piml"

#define PICONF_NAME_S PACKAGE_TARNAME
#define PICONF_NAME_M PICONF_NAME_S
#define PICONF_NAME_L "Processing Instruction Configurator"

namespace piconf {
	class error_t {
	public:
		typedef uint8_t state_t;
		static const state_t
		PICONFM_ERROR = 0x01,
		PICONFS_ERROR = 0x02,
		PARSING_ERROR = 0x04,
		XML_TAG_ERROR = 0x08,
		MAXIMUM_ERROR = 0x10,
		PEER_IO_ERROR = 0x20,
		FILE_IO_ERROR = 0x40,
		SYSCALL_ERROR = 0x80;
		error_t();
		void set(state_t what);
		state_t get(state_t mask = 0xFF);
	private:
		state_t state;
	};
	struct name_t {
		const char* s;
		const char* m;
		const char* l;
	};
	extern error_t error;
	extern const char* SERVICE;
	const size_t MAX_XML_SIZE = 1024*1024;
	const size_t MAX_FILE_SIZE = 1024*1024*50;
	const size_t TIMEOUT = 3;
	const size_t ACTIVATE = 2;
	const size_t RECONNECT_DELAY = 3;
	std::string ts2str(timespec ts);
	std::string time();
	typedef std::string line_t;
	typedef std::deque<line_t> lines_t;
	typedef size_t chunk_t;
	timespec diff(timespec start, timespec end);
	std::string calculate_percentage(size_t value, size_t reference);
	std::string calculate_size(uint64_t size);
	template <typename number_t>
	inline std::string to_string(number_t number) {
		return ((std::ostringstream&)(std::ostringstream() << number)).str();
	}
	template <typename number_t>
	inline number_t to_number(const std::string& string) {
		number_t number;
		if (string[1] == 'x') {
			if (!(std::istringstream(string) >> std::hex >> number))
				throw (std::string)"Cannot convert hex string '"+string+"' to number";
		}
		else {
			if (!(std::istringstream(string) >> number))
				throw (std::string)"Cannot convert string '"+string+"' to number";
		}
		return number;
	}
	void print_version(name_t name, std::ostream& ansi = std::cout);
	void print_help(name_t name, std::ostream& ansi = std::cout);
	void print_info(name_t name, std::ostream& ansi = std::cout);
	std::string parse_args(name_t name, int argc, char** argv);
}

#endif
