/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_M_
#define _PICONF_M_

#include "piconfm.hpp"

#include <string>
#include <vector>
#include <sstream>
#include <string.h>
#include <errno.h>
#include <unistd.h>

piconf::name_t piconf::name = {
	PICONF_NAME_S "m",
	PICONF_NAME_M " master",
	PICONF_NAME_L " (Master)"
};

piconf::lines_t piconf::explode(line_t input, char delimiter, bool empty) {
	lines_t output;
	std::istringstream stream(input);
	line_t line;
	while (std::getline(stream, line, delimiter))
		if (empty or !line.empty())
			output.push_back(line);
	return output;
}

piconf::lines_t piconf::indent(line_t front, lines_t::iterator begin, lines_t::iterator end) {
	lines_t output;
	for (lines_t::iterator line = begin; line != end; ++line)
		output.push_back(front+*line);
	return output;
}

void piconf::lock_terminal(bool lock) {
	static termios oldtc;
	if (lock) {
		if (-1 == tcgetattr(STDIN_FILENO, &oldtc)) {
			piconf::error.set(error_t::SYSCALL_ERROR);
			throw (std::string)"Cannot get terminal attributes: "+strerror(errno);
		}
		termios newtc = oldtc;
		newtc.c_lflag &= ~(ICANON | ECHO);
		if (-1 == tcsetattr(STDIN_FILENO, TCSANOW, &newtc)) {
			piconf::error.set(error_t::SYSCALL_ERROR);
			throw (std::string)"Cannot set terminal attributes: "+strerror(errno);
		}
	}
	else
		if (-1 == tcsetattr(STDIN_FILENO, TCSANOW, &oldtc)) {
			piconf::error.set(error_t::SYSCALL_ERROR);
			throw (std::string)"Cannot reset terminal attributes "+strerror(errno);
		}
}

void piconf::update_terminal(lines_t lines, std::ostream& ansi) {
	static size_t size = 0;
	for (size_t i = 0; i < size; ++i)
		ansi << "\e[A\e[K" << std::flush;
	size = lines.size();
	for (size_t i = 0; i < size; ++i)
		ansi << lines[i] << std::endl;
}

#endif
