/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_CONF_TAG_H
#define _PICONF_CONF_TAG_H

#include "data_tag.hpp"
#include "../../../vio/File.hpp"
#include <vector>
#include <string>
#include <map>

namespace piconf {
	class conf_tag;
}

class piconf::conf_tag : public data_tag {
	typedef uint8_t conf_state_t;
	static const conf_state_t
	PARSED = 0x01;
	conf_state_t conf_state;
	class rb_t {
	public:
		std::string
		path,
		name,
		node,
		data;
	};
	typedef std::deque<rb_t> rb_box_t;
	rb_box_t rb_box;
	class file_t {
	public:
		VIO::File file;
		typedef uint8_t state_t;
		static const state_t
		WRITING = 0x01,
		DONE    = 0x02;
		state_t state;
		std::string* data;
		file_t();
	};
	typedef std::deque<file_t> file_box_t;
	typedef std::map<io_tag*, file_box_t> file_map_t;
	file_map_t file_map;
	void parse_result(pugi::xml_node node, lines_t& lines, bool file_rb = false);
public:
	conf_tag(pugi::xml_node node);
	bool instruct(io_tag* io, bool fail = false, bool exit = false);
	void parse_conf(pugi::xml_node node);
	void parse_response(io_tag* io, pugi::xml_node node);
	void status(lines_t& lines);
	void result(io_tag* io, lines_t& lines);
	bool process();
	void setup();
};

#endif
