/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_FILE_TAG_H
#define _PICONF_FILE_TAG_H

#include "data_tag.hpp"
#include "../../../vio/File.hpp"
#include <map>
#include <vector>
#include <string>
#include <utility>
#include <inttypes.h>

namespace piconf {
	class file_tag;
}

class piconf::file_tag : public data_tag {
	uint8_t state;
	static const uint8_t
	DONE     = 0x01,
	FETCH    = 0x02,
	INVALID  = 0x04,
	READBACK = 0x08;
	static const io_state_t
	INSTRUCTED_FILE = 0x20,
	RECEIVED_FILE   = 0x40,
	WRITING_FILE    = 0x80,
	PROCESSED_FILE  = 0x100;
	size_t file_size;
	char* file_data;
	std::string file_path;
	VIO::File file;
	typedef std::map<io_tag*, VIO::File> file_box_t;
	file_box_t file_box;
public:
	file_tag(pugi::xml_node node);
	void setup();
	bool instruct(io_tag* io, bool fail = false, bool exit = false);
	void parse_response(io_tag* io, pugi::xml_node node);
	void status(lines_t& lines);
	bool complete();
	bool process();
	~file_tag();
};

#endif
