/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_IO_TAG_
#define _PICONF_IO_TAG_

#include "io_tag.hpp"
#include "../../../vio/IO.hpp"

piconf::io_tag::Progress::Progress() {
	done = 0;
	size = 0;
	fail = 0;
}

piconf::io_tag::io_tag(pugi::xml_node node)
: tag(node) {
	data = 0;
	io_state = 0;
}

void piconf::io_tag::status(lines_t& lines) {
	lines.push_back("  NOTE: The endpoint responded with following nodes:");
	size_t size = lines.size();
	for (size_t i = 0; i < data_box.size(); ++i)
		data_box[i]->result(this, lines);
	if (size == lines.size())
		lines.pop_back();
}

bool piconf::io_tag::instruct(bool fail, bool exit) {
	if (!data_box.size())
		return false;
	bool undone = data_box[data]->instruct(this, fail, exit);
	if (exit)
		return false;
	size_t i_size = io->hasToRead();
	size_t o_size = io->hasToWrite();
	if (fail) {
		i_progress.tag.fail += i_progress.pkg.done;
		o_progress.tag.fail += o_progress.pkg.done;

		i_progress.all.fail += i_progress.pkg.done;
		o_progress.all.fail += o_progress.pkg.done;

		i_progress.tag.done = 0;
		o_progress.tag.done = 0;
		i_progress.tag.size = 0;
		o_progress.tag.size = 0;
	}
	if (!fail or !i_progress.tag.size)
		i_progress.tag.size += i_size;
	if (!fail or !o_progress.tag.size)
		o_progress.tag.size += o_size;
	if (!fail or !i_progress.all.size)
		i_progress.all.size += i_size;
	if (!fail or !o_progress.all.size)
		o_progress.all.size += o_size;
	i_progress.pkg.done = 0;
	o_progress.pkg.done = 0;

	i_progress.pkg.size = i_size;
	o_progress.pkg.size = o_size;
	return undone;
}

void piconf::io_tag::subscribe(data_tag* data) {
	data_box.push_front(data);
}

void piconf::io_tag::setup() {
}

void piconf::io_tag::push(const void* query, size_t size, bool copy) {
	io->push(query, size, copy);
}

void piconf::io_tag::fetch(void* value, size_t size, bool text) {
	io->fetch(value, size, text);
}

void piconf::io_tag::subscribe(std::deque<io_tag*>& io_box, bool inherit) {
	io_box.push_back(this);
}

bool piconf::io_tag::process() {
	if (data < data_box.size()) {
		size_t i_left = io->hasToRead();
		size_t o_left = io->hasToWrite();
		size_t i_done = i_progress.pkg.size-i_left;
		size_t o_done = o_progress.pkg.size-o_left;
		size_t i_bytes = i_done-i_progress.pkg.done;
		size_t o_bytes = o_done-o_progress.pkg.done;
		i_progress.pkg.done += i_bytes;
		o_progress.pkg.done += o_bytes;
		i_progress.tag.done += i_bytes;
		o_progress.tag.done += o_bytes;
		i_progress.all.done += i_bytes;
		o_progress.all.done += o_bytes;
		if (!i_left and !o_left) {
			if (!instruct()) {
				++data;
				if (!(data < data_box.size()))
					return false;
				i_progress.tag.done = 0;
				o_progress.tag.done = 0;
				i_progress.tag.size = 0;
				o_progress.tag.size = 0;
				i_progress.tag.fail = 0;
				o_progress.tag.fail = 0;
				instruct();
			}
		}
	}
	else
		return false;
	return true;
}

#endif
