/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_PI_TAG_
#define _PICONF_PI_TAG_

#include "pi_tag.hpp"
#include "../piconfm.hpp"
#include "../../../vio/TCP.hpp"
#include <iostream>
#include <cstring>
#include <typeinfo>

piconf::pi_tag::pi_tag(pugi::xml_node node)
: tag(node) {
	inherit = node.attribute("inherit").as_bool();
}

void piconf::pi_tag::setup() {
	for (pugi::xml_node_iterator r = node.begin(); r != node.end() and r->type() == 2; r = ++r) {
		if (!strcmp(r->name(), "conf")) {
			conf_box.push_back(conf_tag(*r));
			tag_box.push_back(&conf_box.back());
		}
		else
		if (!strcmp(r->name(), "file")) {
			file_box.push_back(file_tag(*r));
			tag_box.push_back(&file_box.back());
		}
		else
		if (!strcmp(r->name(), "shell")) {
			shell_box.push_back(shell_tag(*r));
			tag_box.push_back(&shell_box.back());
		}
		else
		if (!strcmp(r->name(), "daq")) {
			daq_box.push_back(daq_tag(*r));
			tag_box.push_back(&daq_box.back());
		}
		else
		if (!strcmp(r->name(), "tcp")) {
			tcp_box.push_back(tcp_tag(*r));
			tag_box.push_back(&tcp_box.back());
		}
		else {
			throw (std::string)"Invalid PI tag: "+r->name();
		}
	}
	for (tag_box_t::iterator it = tag_box.begin(); it != tag_box.end(); ++it)
		(*it)->setup();
	for (tag_box_t::reverse_iterator it = tag_box.rbegin(); it != tag_box.rend(); ++it)
		(*it)->subscribe(io_box, inherit);
}

void piconf::pi_tag::schedule(const timespec* interval, const timespec* realtime, const timespec* accuracy) {
	timespec realtime_var, accuracy_var;
	if (!realtime) {
		realtime = &realtime_var;
		//clock_gettime(CLOCK_REALTIME, &realtime_var);
	}
	if (!accuracy) {
		accuracy = &accuracy_var;
		//clock_getres(CLOCK_REALTIME, &accuracy_var);
	}
	for (tag_box_t::reverse_iterator it = tag_box.rbegin(); it != tag_box.rend(); ++it)
		(*it)->schedule(interval, realtime, accuracy);
}

bool piconf::pi_tag::process() {
	bool undone = false;
	try {
		for (tag_box_t::iterator it = tag_box.begin(); it != tag_box.end(); ++it)
			if ((*it)->process())
				undone = true;
		if (undone)
			VIO::IO::process();
	}
	catch (VIO::File::Error& error) {
		piconf::error.set(error_t::FILE_IO_ERROR | error_t::PICONFM_ERROR);
		throw error.what();
	}
	catch (VIO::IO::Error& error) {
		if (error.instance) {
			if (error.instance == VIO::IO::timer)
				throw error;
			((tag*)error.instance)->handle(error.code, error.what());
		}
		else {
			piconf::error.set(error_t::SYSCALL_ERROR);
			throw error.what();
		}
		undone = true;
	}
	return undone;
}

void piconf::pi_tag::status(lines_t& lines) {
	for (tag_box_t::iterator it = tag_box.begin(); it != tag_box.end(); ++it)
		(*it)->status(lines);
}

piconf::pi_tag::~pi_tag() {
}

#endif
