/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_TCP_TAG_
#define _PICONF_TCP_TAG_

#include "tcp_tag.hpp"

piconf::tcp_tag::tcp_tag(const char* tpl_file)
: io_tag(tpl_file) {
	tcp_state = 0;
	pugi::xml_attribute service = tpl.child("io").attribute("service");
	if (!service)
		service.set_value(SERVICE);
	tcp.listen(service.value());
	log << time() << "Listening on '" << service.value() << "' with timeout of " << timeout.tv_sec << "s" << std::endl;
}

bool piconf::tcp_tag::process() {
	if (tcp_state & CONNECTED) {
		bool isConnected = false;
		try {
			isConnected = ((VIO::TCP*)queued.front())->isConnected(timeout);
		}
		catch (VIO::IO::Error& error) {
			handle(error);
		}
		if (isConnected and !(state & CLOSE) and !(state & HOOKING_UP)) {
		}
		else {
			delete tag;
			tag = 0;
			delete[] xml;
			xml = 0;
			if (!queued.size())
				throw (std::string)"Connected but queue is empty";
			log << time();
			if (state & HOOKING_UP)
				log << "Hooked up master";
			else {
				delete queued.front();
				log << "Closing master";
			}
			queued.pop_front();
			log << " '" << host << "'. Left in queue: " << queued.size() << std::endl;
			state = 0;
			tcp_state = 0;
			return true;
		}
	}
	else {
		if (tcp.clients.size()) {
			queued.insert(queued.end(), tcp.clients.begin(), tcp.clients.end());
			tcp.clients.clear();
			tcp_state |= CONNECTED;
			try {
				host = ((VIO::TCP*)queued.front())->host();
			}
			catch (VIO::IO::Error& error) {
				host = error.what();
			}
			log << time() << "Processing master '" << host << "'. Left in queue: " << (queued.size()-1) << std::endl;
			return true;
		}
	}
	for (hooked_t::iterator hook = hooked.begin(); hook != hooked.end(); ++hook)
		for (data_tag::hooked_t::ios_t::iterator io = (*hook)->ios.begin(); io != (*hook)->ios.end(); ++io) {
			if (!*io) {
				log << time() << "Skipping already deleted, hooked io" << std::endl;
				continue;
			}
			bool isConnected = false;
			try {
				isConnected = ((VIO::TCP*)*io)->isConnected(timeout);
			}
			catch (VIO::IO::Error& error) {
				handle(error, *io);
			}
			if (!isConnected) {
				delete *io;
				*io = 0;
				log << time() << "Lost hooked up master" << std::endl;
			}
		}
	return io_tag::process();
}

void piconf::tcp_tag::handle(VIO::IO::Error& error, VIO::IO* io) {
	if (io) {
		io->clear();
		log << time() << "Handling hook exception: " << error.what() << std::endl;
	}
	else {
		tcp_state |= CONNECTED;
		log << time() << "Handling exception: " << error.what() << std::endl;
	}
}

piconf::tcp_tag::~tcp_tag() {
}

#endif
