/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _AXI_H
#define _AXI_H

#include <sys/types.h>
#include <inttypes.h>
#include <unistd.h>
#include <time.h>
#include <string>
#include <sstream>
#include <exception>

#define AXI_HSX_DELAY 4
#define AXI_ZDX_DELAY 10

#define AXI_DEVICE "/dev/mem"

namespace VIO {
	class AXI;
}

class VIO::AXI {
	int fd;
	AXI* hsi, * hso, * nxt;
	uint8_t done, idle;
	volatile uint32_t* data;
	inline void construct() {
		device = AXI_DEVICE;
		hsx_delay = AXI_HSX_DELAY;
		zdx_delay = AXI_ZDX_DELAY;
	}
	inline static int nsleep(const long nsec, const time_t sec = 0) {
		timespec req = { sec, nsec };
		return nanosleep(&req, 0);
	}
	inline void wait_until(uint8_t what) {
		while (!(*(hsi->data) & what))
			nsleep(hsx_delay);
	}
	inline void wait_until_not(uint8_t what) {
		while (*(hsi->data) & what)
			nsleep(hsx_delay);
	}
	inline void set(uint8_t what) {
		*(hso->data) |= what;
	}
	inline void unset(uint8_t what) {
		*(hso->data) &= ~what;
	}
public:
	class Error : public std::exception {
	public:
		std::string context;
		const void* instance;
		int code;
		Error(const char* context, const char* message = 0, const void* instance = 0, int code = 0);
		virtual const char* what() const throw();
		virtual ~Error() throw();
	};
	size_t hsx_delay;
	size_t zdx_delay;
	std::string device;
	AXI();
	AXI(off_t base_addr, AXI* hsi_axi = 0, AXI* hso_axi = 0, uint8_t done_bit = 0, uint8_t idle_bit = 0);
	void open(off_t base_addr, AXI* hsi_axi = 0, AXI* hso_axi = 0, uint8_t done_bit = 0, uint8_t idle_bit = 0);
	void next(AXI* nxt);
	void read(uint32_t* word);
	void write(const uint32_t* word);
	void close();
	~AXI();
};

#endif
