/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2012, 2013  Rouven Spreckels  <nevuor@piconf.org>           *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _VIO_TCP_H
#define _VIO_TCP_H

#include "IO.hpp"

#include <netdb.h>
#include <list>
#include <string>

namespace VIO {
	class TCP;
}

class VIO::TCP : public IO {
public:
	typedef std::list<TCP*> Clients;
protected:
	Clients::reverse_iterator it;
	struct sockaddr_storage addr;
	struct addrinfo *address;
	socklen_t addrlen;
	typedef char State;
	State state;
	static const State
	LISTENING  = 0x01,
	ACCEPTING  = 0x02,
	ACCEPTABLE = 0x04,
	CONNECTING = 0x08,
	CONNECTED  = 0x10;
	virtual void readable();
	virtual void writable();
	virtual void read();
	virtual void write();
	virtual void timewise();
	void connecting();
	static void timing(TCP* tcp, timespec span);
public:
	typedef uint8_t error_t;
	static const error_t
	SOCKETPAIR  = 13,
	GETPEERNAME = 14,
	GETADDRINFO = 15,
	GETNAMEINFO = 16,
	BIND        = 17,
	LISTEN      = 18,
	UNAVAILABLE = 19,
	TIMEOUT     = 20;
	static void pair(TCP* tcp0, TCP* tcp1);
	static void forkTimer(TCP* tcp, timespec span);
	Clients clients;
	class Error : public IO::Error {
	public:
		Error(const char* context, const char* message = 0, const void* instance = 0, int code = 0);
	};
	TCP();
	void listen(std::string service, bool reuse = true, bool accepting = true);
	TCP::Clients::reverse_iterator accept();
	void connect(std::string host, std::string service);
	bool isConnected();
	bool isConnected(timespec timeout);
	std::string host();
	virtual void clear();
	virtual ~TCP();
};

#endif
